### legacy DNS bindings - DEPRECATED ###

from .dns_providers import AuroraDns  # noqa: F401
from .dns_providers import CloudFlareDns  # noqa: F401
from .dns_providers import AcmeDnsDns  # noqa: F401
from .dns_providers import AliyunDns  # noqa:F401
from .dns_providers import HurricaneDns  # noqa:F401
from .dns_providers import RackspaceDns  # noqa:F401
from .dns_providers import DNSPodDns  # noqa:F401
from .dns_providers import DuckDNSDns  # noqa:F401
from .dns_providers import ClouDNSDns  # noqa:F401
from .dns_providers import Route53Dns  # noqa:F401
from .dns_providers import PowerDNSDns  # noqa:F401
from .dns_providers import GandiDns  # noqa:F401
